<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:eb="http://www.ebenoit.info/data"
	xmlns:dc="http://purl.org/dc/elements/1.1/">

	<xsl:output method="xml"
			omit-xml-declaration="no"
			indent="yes"
			media-type="application/xhtml+xml"
			doctype-system="http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"
			doctype-public="-//W3C//DTD XHTML 1.1//EN" />


	<!-- *********************************************************************************************** -->
	<!-- * PARAMETERS                                                                                  * -->
	<!-- *********************************************************************************************** -->

	<!-- Requested page (defaulting to 'main') -->
	<xsl:param name="reqPage">main</xsl:param>

	<!-- Requested language (defaulting to 'en') -->
	<xsl:param name="reqLang">en</xsl:param>

	<!-- Path to CSS files and other data -->
	<xsl:param name="cssPath" />

	<!-- Path to the generator script -->
	<xsl:param name="scriptPath" />


	<!-- *********************************************************************************************** -->
	<!-- * VARIABLES                                                                                   * -->
	<!-- *********************************************************************************************** -->

	<!-- Root of the data file -->
	<xsl:variable name="d" select="/eb:data" />

	<!-- Actual language to use -->
	<xsl:variable name="lang">
		<xsl:choose>
			<xsl:when test="$d/eb:contents[@lang = $reqLang]/eb:page/eb:page[@id = $reqPage]">
				<xsl:value-of select="$reqLang" />
			</xsl:when>
			<xsl:when test="$d/eb:contents[@lang = $reqLang]/eb:page[@id = $reqPage]">
				<xsl:value-of select="$reqLang" />
			</xsl:when>
			<xsl:otherwise>en</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<!-- Actual page to display -->
	<xsl:variable name="page">
		<xsl:choose>
			<xsl:when test="$d/eb:contents[@lang = $lang]/eb:page/eb:page[@id = $reqPage]">
				<xsl:value-of select="$reqPage" />
			</xsl:when>
			<xsl:when test="$d/eb:contents[@lang = $lang]/eb:page[@id = $reqPage]">
				<xsl:value-of select="$reqPage" />
			</xsl:when>
			<xsl:otherwise>main</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<!-- Root for the language-specific definitions -->
	<xsl:variable name="ld" select="$d/eb:languages/eb:language[@id=$lang]" />

	<!-- Root for the contents of the page -->
	<xsl:variable name="cd" select="$d/eb:contents[@lang=$lang]/eb:page[@id=$page]|$d/eb:contents[@lang=$lang]/eb:page/eb:page[@id=$page]" />

	<!-- Page author, if there is one. If there is no page-specific author, use the site-wise value.
	   - If there is no site-wise author, don't send any author metadata.
	   -->
	<xsl:variable name="page-author">
		<xsl:choose>
			<xsl:when test="$d/eb:contents[@lang = $lang]/eb:page[@id = $reqPage]/@author">
				<xsl:value-of select="$d/eb:contents[@lang = $lang]/eb:page[@id = $reqPage]/@author" />
			</xsl:when>
			<xsl:when test="$d/eb:author">
				<xsl:value-of select="$d/eb:author" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="false" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<!-- Is the menu enabled? -->
	<xsl:variable name="menu-enabled"
				  select="$d/eb:contents[@lang = $lang]/eb:page/@autogen or $d/eb:contents[@lang = $lang]/eb:page/eb:page" />


	<!-- *********************************************************************************************** -->
	<!-- * XSL TEMPLATES                                                                               * -->
	<!-- *********************************************************************************************** -->

	<xsl:template match="/">
		<html>
			<xsl:attribute name="xml:lang">
				<xsl:value-of select="$lang" />
			</xsl:attribute>
			<head>
				<meta http-equiv="Content-Type" content="application/xhtml+xml;charset=utf-8" />
				<xsl:if test="$page-author">
					<meta name="author">
						<xsl:attribute name="content"><xsl:value-of select="$page-author" /></xsl:attribute>
					</meta>
				</xsl:if>
				<link rel="stylesheet" type="text/css">
					<xsl:attribute name="href"><xsl:value-of select="$cssPath" />css/style.css</xsl:attribute>
				</link>

				<xsl:if test="$d/eb:feeds or $menu-enabled">
					<script type='text/javascript' charset='utf-8'>
						<xsl:attribute name="src"><xsl:value-of select="$cssPath" />js/jquery.js</xsl:attribute>
						<xsl:text disable-output-escaping="yes">&lt;!--
						//--&gt;</xsl:text>
					</script>
				</xsl:if>
				<xsl:if test="$d/eb:feeds">
					<script type='text/javascript' charset='utf-8'>
						<xsl:attribute name="src"><xsl:value-of select="$cssPath" />js/feeds.js</xsl:attribute>
						<xsl:text disable-output-escaping="yes">&lt;!--
						//--&gt;</xsl:text>
					</script>
				</xsl:if>
				<xsl:if test="$menu-enabled">
					<script type='text/javascript' charset='utf-8'>
						<xsl:attribute name="src"><xsl:value-of select="$cssPath" />js/menu.js</xsl:attribute>
						<xsl:text disable-output-escaping="yes">&lt;!--
						//--&gt;</xsl:text>
					</script>
					<link rel="stylesheet" type="text/css">
						<xsl:attribute name="href"><xsl:value-of select="$cssPath" />css/menu.css</xsl:attribute>
					</link>
				</xsl:if>

				<title>
					<xsl:value-of select="$ld/eb:string[@id='title']" /> -
					<xsl:value-of select="$cd/@title" />
				</title>
			</head>
			<body>
				<xsl:call-template name="header-block" />
				<xsl:call-template name="other-data" />
				<xsl:call-template name="contents" />
				<xsl:if test="$menu-enabled">
					<div style="display:none">
						<img id="js-menu-open" alt="(internal)">
							<xsl:attribute name="src">
									<xsl:value-of select="concat($cssPath,'js/menu-open.png')" />
							</xsl:attribute>
						</img>
						<img id="js-menu-closed" alt="(internal)">
							<xsl:attribute name="src">
									<xsl:value-of select="concat($cssPath,'js/menu-closed.png')" />
							</xsl:attribute>
						</img>
					</div>
				</xsl:if>
			</body>
		</html>
	</xsl:template>

	<xsl:template name="header-block">
		<div id="header">
			<h1><xsl:value-of select="$ld/eb:string[@id='title']" /></h1>
			<h2><xsl:value-of select="$cd/@title" /></h2>
			<div id="language">
				<xsl:for-each select="$d/eb:languages/eb:language">
					<xsl:apply-templates select="." />
				</xsl:for-each>
			</div>
		</div>
	</xsl:template>

	<xsl:template name="other-data">
		<div id="other-data">

			<div id="pages">
				<div id="pages-title"><xsl:value-of select="$ld/eb:string[@id='pages']" /></div>
				<xsl:for-each select="$d/eb:contents[@lang=$lang]/eb:page[not(@hidden)]">
					<xsl:apply-templates select="." />
				</xsl:for-each>
			</div>

			<xsl:if test="$d/eb:feeds">
				<xsl:call-template name="rss-feeds" />
			</xsl:if>
			<xsl:if test="$d/eb:history[@lang=$lang]">
				<xsl:apply-templates select="$d/eb:history[@lang=$lang]" />
			</xsl:if>
			<div style="text-align:center; width:100%;padding: 20px 0px 0px 0px">
				<a href="http://validator.w3.org/check?uri=referer">
					<img style="border:0;height:31px;width:88px" alt="Valid XHTML 1.1">
						<xsl:attribute name="src"><xsl:value-of select="$cssPath" />pics/valid-xhtml11-blue.png</xsl:attribute>
					</img>
				</a> 
				<a href="http://jigsaw.w3.org/css-validator/">
					<img style="border:0;width:88px;height:31px" alt="Valid CSS!">
						<xsl:attribute name="src"><xsl:value-of select="$cssPath" />pics/vcss-blue.png</xsl:attribute>
					</img>
				</a>
  			</div>
		</div>
	</xsl:template>
	
	<xsl:template name="rss-feeds">
		<div id="rss-feeds">
			<div id="feeds-title"><xsl:value-of select="$ld/eb:string[@id='rss-feeds']" /></div>
			<xsl:for-each select="$d/eb:feeds/eb:feed">
				<xsl:apply-templates select="." />
			</xsl:for-each>
		</div>
	</xsl:template>
	
	<xsl:template match="eb:feed">
		<xsl:variable name="feed" select="document(@from)" />
		<xsl:variable name="hide-link" select="(@hide-link and @hide-link = '1')" />

		<xsl:if test="$feed and $feed/rss">
			<div class="rss-feed">
				<xsl:attribute name="id"><xsl:value-of select="concat('feed-', @id)" /></xsl:attribute>
				<div class="feed-title">
					<xsl:value-of select="$feed/rss/channel/title" />
				</div>
				<div class="feed-container">
					<xsl:if test="$feed/rss/channel/description">
						<div class="feed-description">
							<xsl:value-of select="$feed/rss/channel/description" />
						</div>
					</xsl:if>
					<div class="feed-contents">
						<xsl:for-each select="$feed/rss/channel/item[position() &lt;= 5]">
							<div class="feed-item">
								<div class="feed-item-title">
									<xsl:choose>
										<xsl:when test="$hide-link or not(link)">
											<xsl:value-of select="title" disable-output-escaping="no" />
										</xsl:when>
										<xsl:otherwise>
											<a>
												<xsl:attribute name="href">
													<xsl:value-of select="link" />
												</xsl:attribute>
												<xsl:value-of select="title" disable-output-escaping="yes" />
											</a>
										</xsl:otherwise>
									</xsl:choose>
								</div>
								<div class="feed-item-date">
									<xsl:if test="dc:creator">
										<xsl:value-of select="dc:creator" /> /
									</xsl:if>
									<xsl:value-of select="pubDate" />
								</div>
							</div>
						</xsl:for-each>
					</div>
				</div>
			</div>
		</xsl:if>
	</xsl:template>

	<xsl:template name="contents">
		<div id="contents">
			<xsl:choose>
				<xsl:when test="$cd[@datafile]">
					<xsl:for-each select="document($cd/@datafile)/eb:ext-page/node()">
						<xsl:apply-templates select="." />
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>
					<xsl:for-each select="$cd/node()">
						<xsl:apply-templates select="." />
					</xsl:for-each>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	</xsl:template>

	<xsl:template match="eb:language">
		<span>
			<xsl:attribute name="id">ls-<xsl:value-of select="@id" /></xsl:attribute>
			<xsl:choose>
				<xsl:when test="@id != $lang">
					<a>
						<xsl:attribute name="href">
							<xsl:value-of select="concat($scriptPath,'/',@id,'/',$page)" />
						</xsl:attribute>
						<xsl:attribute name="title">
							<xsl:variable name="plang" select="@id" />
							<xsl:value-of select="concat($d/eb:contents[@lang=$plang]/eb:page[@id=$page]/@title,' (',eb:langname,')')" />
						</xsl:attribute>
						<xsl:value-of select="eb:langname" />
					</a>
				</xsl:when>
				<xsl:otherwise>
					<strong><xsl:value-of select="eb:langname" /></strong>
				</xsl:otherwise>
			</xsl:choose>
		</span>
	</xsl:template>

	<xsl:template match="eb:page">
		<xsl:variable name="is-active" select="@id = $page" />
		<xsl:variable name="is-sub-page" select="../../eb:page" />

		<span>
			<xsl:attribute name="id">pg-<xsl:value-of select="@id" /></xsl:attribute>

			<xsl:if test="$menu-enabled">
				<xsl:variable name="a-class">
					<xsl:if test="$is-active">active</xsl:if>
				</xsl:variable>

				<xsl:variable name="l-class">
					<xsl:choose>
						<xsl:when test="$is-sub-page">menu-level-2</xsl:when>
						<xsl:otherwise>menu-level-1</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:variable name="o-class">
					<xsl:choose>
						<xsl:when test="$is-sub-page or ($is-active and not(eb:page or @autogen))"></xsl:when>
						<xsl:when test="$is-active or eb:page[@id = $page]">open</xsl:when>
						<xsl:otherwise>closed</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:attribute name="class">
					<xsl:value-of select="concat( $l-class , ' ' , $a-class , ' ' , $o-class )" />
				</xsl:attribute>
			</xsl:if>

			<xsl:choose>
				<xsl:when test="@id != $page">
					<a>
						<xsl:attribute name="href">
							<xsl:value-of select="concat($scriptPath,'/',$lang,'/',@id)" />
						</xsl:attribute>
						<xsl:attribute name="title">
							<xsl:value-of select="@title" />
						</xsl:attribute>
						<xsl:value-of select="@title" />
					</a>
				</xsl:when>
				<xsl:otherwise>
					<strong><xsl:value-of select="@title" /></strong>
				</xsl:otherwise>
			</xsl:choose>
		</span>

		<xsl:if test="$menu-enabled and not($is-sub-page)">

			<xsl:if test="@autogen">
				<xsl:variable name="autogen" select="@autogen" />
				<xsl:variable name="id" select="@id" />
				<xsl:choose>
					<xsl:when test="@datafile">
						<xsl:for-each select="document(@datafile)/eb:ext-page/*[@class = $autogen and @id]">
							<span class="menu-level-2">
								<a>
									<xsl:attribute name="href">
										<xsl:value-of select="concat($scriptPath,'/',$lang,'/',$id,'#',@id)" />
									</xsl:attribute>
									<xsl:value-of select="." />
								</a>
							</span>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<xsl:for-each select="*[@class = $autogen and @id]">
							<span class="menu-level-2">
								<a>
									<xsl:attribute name="href">
										<xsl:value-of select="concat($scriptPath,'/',$lang,'/',$id,'#',@id)" />
									</xsl:attribute>
									<xsl:value-of select="." />
								</a>
							</span>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>

			<xsl:if test="eb:page">
				<xsl:apply-templates select="eb:page" />
			</xsl:if>
		</xsl:if>
	</xsl:template>


	<xsl:template match="eb:history">
		<div id="history">
			<div id="history-title"><xsl:value-of select="$ld/eb:string[@id='history']" /></div>
			<xsl:for-each select="eb:entry[position() &lt;= 5]">
				<div>
					<span class="history-date"><xsl:value-of select="@date" /></span>
					<span class="history-text"><xsl:apply-templates select="node()" /></span>
				</div>
			</xsl:for-each>
			<div id="history-complete"><a>
				<xsl:attribute name="href"><xsl:value-of select="$scriptPath" />/<xsl:value-of select="$lang" />/history</xsl:attribute>
				<xsl:attribute name="title"><xsl:value-of select="$ld/eb:string[@id='complete-history']" /></xsl:attribute>
				<xsl:value-of select="$ld/eb:string[@id='complete-history']" />
			</a></div>
		</div>
	</xsl:template>

	<xsl:template match="eb:complete-history">
		<ul>
			<xsl:for-each select="$d/eb:history[@lang=$lang]/eb:entry">
				<li>
					<span style="text-decoration:underline"><xsl:value-of select="@date" /></span><br/>
					<xsl:apply-templates select="node()" />
				</li>
			</xsl:for-each>
		</ul>
	</xsl:template>

	<xsl:template match="eb:linkto">
		<a>
			<xsl:attribute name="href">
				<xsl:choose>
					<xsl:when test="@page"><xsl:value-of select="concat($scriptPath,'/',$lang,'/',@page)" /></xsl:when>
					<xsl:when test="@relative"><xsl:value-of select="concat($cssPath,@relative)" /></xsl:when>
					<xsl:otherwise>/invalid-link</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:choose>
					<xsl:when test="@page">
						<xsl:variable name="tpage"><xsl:value-of select="@page" /></xsl:variable>
						<xsl:value-of select="$d/eb:contents[@lang=$lang]/eb:page[@id=$tpage]/@title" />
					</xsl:when>
					<xsl:when test="@title"><xsl:value-of select="@title" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="$ld/eb:string[@id='untitled-link']" /></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:apply-templates select="@style|@class|@id" />
			<xsl:apply-templates select="node()" />
		</a>
	</xsl:template>

	<xsl:template match="eb:image">
		<img>
			<xsl:attribute name="src">
				<xsl:choose>
					<xsl:when test="@external"><xsl:value-of select="@external" /></xsl:when>
					<xsl:when test="@relative"><xsl:value-of select="concat($cssPath,@relative)" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="concat($cssPath,'/error.png')" /></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:attribute name="alt">
				<xsl:choose>
					<xsl:when test="@title"><xsl:value-of select="@title" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="$ld/eb:string[@id='untitled-image']" /></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:apply-templates select="@style|@class|@id" />
		</img>
	</xsl:template>

	<xsl:template match="@*|node()" priority="-1">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()"/>
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>
